#include "chadenemy.h"

CChadEnemy::CChadEnemy(SceneManager* scenemgr) {
	m_SceneMgr = scenemgr;

	static int EnemyNum = 0;

	sprintf (m_EnemyName, "Robot %d", ++EnemyNum);
	m_Entity = m_SceneMgr->createEntity(m_EnemyName, "robot.mesh");

	SetAnimation("Walk", true);

	m_Node = m_SceneMgr->getRootSceneNode()->createChildSceneNode(m_EnemyName);
	m_Node->attachObject(m_Entity);
	m_Node->scale(0.25,0.25,0.25);

	m_Destination = Vector3::ZERO;

	m_WatchingPlayer = true;

	m_IsDead = false;

	m_Speed = 5;
}

CChadEnemy::~CChadEnemy() {

}

void CChadEnemy::SetPos(Vector3 dest) {
	m_Node->translate(dest);

	Vector3 ourpos = m_Node->getPosition();
	Ray ourray(Vector3(ourpos.x, 1000.0f, ourpos.z), Vector3::NEGATIVE_UNIT_Y);

	RaySceneQuery* query = m_SceneMgr->createRayQuery( Ray() );

	query->setRay(ourray);

	RaySceneQueryResult &result = query->execute();
        RaySceneQueryResult::iterator iter = result.begin();

	if (iter != result.end() && iter->worldFragment) {
		float terrainheight = iter->worldFragment->singleIntersection.y;
		if (terrainheight != ourpos.y) m_Node->setPosition(ourpos.x, terrainheight, ourpos.z);
	}

	m_SceneMgr->destroyQuery(query);
}

void CChadEnemy::Update(Real time) {
	m_AnimationState->addTime(time);

	if (m_IsDead) return;

	Vector3 campos = m_SceneMgr->getSceneNode("camnode")->getPosition();
	Vector3 ourpos = m_Node->getPosition();

	Vector3 playerdist = ourpos - campos;
	if (playerdist.normalise() < 100) {
		if (!m_WatchingPlayer) {
			SetAnimation("Idle", true);

			m_WatchingPlayer = true;
		}

		Vector3 orient = m_Node->getOrientation() * Vector3::UNIT_X;
		Vector3 dir = campos - m_Node->getPosition();
		dir.y = 0;
		Ogre::Quaternion quat = orient.getRotationTo(dir);
		m_Node->rotate(quat);
	} else {
		if (m_Destination == Vector3::ZERO) {
			// get a new destination
			m_Destination = Vector3(rand() % 1000,0, rand() % 1000);
			m_Direction = m_Destination - m_Node->getPosition();

		} else {
			// how far are we from the goal?
			ourpos.y = 0.0f; // this is calculated automatically, and should be ignored for distance purposes
			m_Direction = m_Destination - ourpos;

			// the previous length of the vector is returned when we call normalise(), so we need to catch that
			Real dist = m_Direction.normalise();

			Real movespeed = m_Speed * time;

			if (dist < movespeed) {
				// we're (basically) at the destination; pick a new one!
				m_Destination = Vector3::ZERO;		
			} else {
				// need to inch closer to the destination
				Vector3 newpos = m_Direction * movespeed;
				SetPos(newpos);
			}

			Vector3 orient = m_Node->getOrientation() * Vector3::UNIT_X;
			Ogre::Quaternion quat = orient.getRotationTo(m_Direction);
			m_Node->rotate(quat);


		}

		if (m_WatchingPlayer) {
			SetAnimation("Walk", true);
			m_WatchingPlayer = false;
		}
	}
}

void CChadEnemy::SetAnimation(String animation, bool loop) {
	m_AnimationState = m_Entity->getAnimationState(animation);
	m_AnimationState->setLoop(loop);
	m_AnimationState->setEnabled(true);
}

void CChadEnemy::Hit() {
	if (!m_IsDead) {
		m_AnimationState->setEnabled(false);
		SetAnimation("Die", false);
		m_IsDead = true;
	}
}

